<?php
namespace Controller;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Util\Messages;

class ProjectClassification extends \Generated\Controller\Classification {

    /**
     * @var \Util\Utilities
     */
    private $utilities;

    /**
     * @var \Service\Classification
     */
    private $classificationService;

    public function __construct(Manager $dataSourceManager, \Util\Utilities $utilities,
                                \Service\Classification $classificationService){
        $this->utilities = $utilities;
        parent::__construct($dataSourceManager);
        $this->classificationService = $classificationService;
    }

    /**
     * Rewriting our find() method and setting a condition to FilterCriteria
     *
     * @param DTO\Request\Filter $request   A DTO\Request with a FilterCriteria
     * @param DTO\Response $response        A DTO\Response to be added with DataSets
     */
    public function find(DTO\Request\Filter $request, DTO\Response $response){
        $request->getFilterCriteria()->addCondition("ORGANIZATION", "=", $this->utilities->getCurrentOrganizationId());
        $request->getFilterCriteria()->addCondition("IS_PROJECT", "=", "Y");
        parent::find($request, $response);
        $classifications = $response->getDataSets();
        $rows =  $classifications[0]->getRows();
        foreach ($rows as &$row) {
            $row['ALLOWED_PROJECT_TYPE'] = $this->utilities->setSelectMultiInQuery($row['ALLOWED_PROJECT_TYPE']);
            $row['NOT_ALLOWED_PROJECT_TYPE'] = $this->utilities->setSelectMultiInQuery($row['NOT_ALLOWED_PROJECT_TYPE']);
        }
        $response->setDataSets([new DataSet('projectClassification', $rows)]);
    }

    /**
    * Rewriting our save() method and setting a
    *
    * @param DTO\Request\DataSet $dataSet   A DTO\Request with a DataSets
    * @param DTO\Response $response         A DTO\Response to be added with DataSets
    */
    public function save(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $isProject = "Y";
            $isTask = "N";
            $id = $row["ID"];
            $label = $row["LABEL"];
            $allowedProjectType = empty($row["ALLOWED_PROJECT_TYPE"]) ? array() : $row["ALLOWED_PROJECT_TYPE"];
            $notAllowedProjectType = empty($row["NOT_ALLOWED_PROJECT_TYPE"]) ? array(): $row["NOT_ALLOWED_PROJECT_TYPE"];
            $control = empty($row["CONTROL"]) ? null : $row["CONTROL"];

            $dataSet = $this->classificationService->saveClassification($id, $label, $isProject, $isTask, $control,
                $allowedProjectType, array(), $notAllowedProjectType,  array());
            $response->addDataset(new DataSet('projectClassification', $dataSet));
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage() . ' Code:' . $e->getCode());
        }
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $row              = $request->getDataSet()->getRows()[0];
            $classificationId = $row['ID'];
            $this->classificationService->deleteClassification($classificationId);
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_REMOVED));
        } catch (\Exception $ex) {
            throw new \Exception($e->getMessage() . ' Code:' . $e->getCode());
        }
    }
}

