<?php
namespace Zeedhi\DevOps\Model\Configurationmanager\Repositories;

use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;


class SubreleaseProductFiles extends \Doctrine\ORM\EntityRepository {
    /**
     * @param $productId
     * @param $subreleaseId
     * @return array
     */
    public function findSelectedProductFilesByProductId($productId, $subreleaseId, $version, $revision, $filter = null, $limit = null, $offset = null) {
        $query = <<<SQL
        SELECT *
          FROM (SELECT NVL(PV.VERSION, '0') AS VERSION, NVL(PV.REVISION, '0') AS REVISION, PF.PRODUCT_VERSION_ID,
                       NVL(PV.VERSION, '0') || '.' || NVL(PV.REVISION, '0') AS PRODUCT_VERSION,
                       CASE WHEN SP.SUBRELEASE_ID IS NOT NULL OR
                                 PF.IS_REQUIRED = 'Y' OR
                                 PF.TYPE = 'O' OR
                                 NVL(PS.SP_PRODUCT_FILES_ID, PS.PF_PRODUCT_FILES_ID) IS NOT NULL
                            THEN 'Y'
                            ELSE 'N'
                        END AS IS_SELECTED,
                       PF.ID AS PRODUCT_FILES_ID, PF.NAME AS PRODUCT_FILES_NAME,
                       PF.RELATIVE_PATH, PF.TYPE, PF.IS_REQUIRED, PF.FILE_NAME, PF.TEXT,
                       PR.NAME AS PRODUCT_NAME, PR.ID AS PRODUCT_ID, SR.NAME AS SUBRELEASE_NAME,
                       SP.ID, SP.SUBRELEASE_ID
                  FROM PRODUCT PR
            INNER JOIN PRODUCT_FILES PF ON  PF.PRODUCT_ID = PR.ID
             LEFT JOIN SUBRELEASE_PRODUCT_FILES SP ON SP.PRODUCT_FILES_ID = PF.ID AND SP.SUBRELEASE_ID = :SUBRELEASE
             LEFT JOIN SUBRELEASE SR ON SR.ID = SP.SUBRELEASE_ID
             LEFT JOIN PRODUCT_VERSION PV ON PV.ID = PF.PRODUCT_VERSION_ID
             LEFT JOIN (SELECT PF.FILE_NAME, PF.RELATIVE_PATH, PF.PRODUCT_ID,
                               MIN(SP.PRODUCT_FILES_ID) AS SP_PRODUCT_FILES_ID, MIN(PF.ID) AS PF_PRODUCT_FILES_ID
                          FROM PRODUCT_FILES PF
                     LEFT JOIN SUBRELEASE_PRODUCT_FILES SP ON SP.PRODUCT_FILES_ID = PF.ID AND SP.SUBRELEASE_ID = :SUBRELEASE
                         WHERE PF.PRODUCT_ID = :PRODUCT
                           AND PF.IS_REQUIRED = 'O'
                      GROUP BY PF.FILE_NAME, PF.RELATIVE_PATH, PF.PRODUCT_ID
                        HAVING COUNT(*) > 0
                       ) PS ON PF.ID = NVL(PS.SP_PRODUCT_FILES_ID, PS.PF_PRODUCT_FILES_ID)
                           AND PF.PRODUCT_ID = PS.PRODUCT_ID
            INNER JOIN (SELECT PF.FILE_NAME, PF.RELATIVE_PATH,
                               NVL(MAX(PV.VERSION), '0') AS VERSION, NVL(MAX(PV.REVISION), '0') AS REVISION
                          FROM PRODUCT_FILES PF
                          LEFT JOIN PRODUCT_VERSION PV ON PV.ID = PF.PRODUCT_VERSION_ID
                         WHERE PF.PRODUCT_ID = :PRODUCT
                           AND (PV.VERSION  <= :VERSION  OR PF.PRODUCT_VERSION_ID IS NULL)
                           AND (PV.REVISION <= :REVISION OR PF.PRODUCT_VERSION_ID IS NULL)
                      GROUP BY PF.FILE_NAME, PF.RELATIVE_PATH
                       ) VR ON VR.VERSION  = NVL(PV.VERSION, '0')
                           AND VR.REVISION = NVL(PV.REVISION, '0')
                           AND VR.FILE_NAME = PF.FILE_NAME
                           AND VR.RELATIVE_PATH = PF.RELATIVE_PATH
                 WHERE PR.ID = :PRODUCT
              ORDER BY PF.TYPE, PF.FILE_NAME, PF.ID)
         WHERE 1 = 1
           $filter
SQL;
        $params = array (
            'PRODUCT'    => $productId,
            'SUBRELEASE' => $subreleaseId,
            'VERSION'    => $version,
            'REVISION'   => $revision
        );
        $query = DevOpsUtilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
}